@@ -65,6 +65,7 @@
 
 #define BITS_PER_LONG (sizeof(long) * 8)
 #define NBITS(x) ((((x)-1)/BITS_PER_LONG)+1)
+#define NBYTES(x) (NBITS(x) * sizeof(long))
 #define OFF(x)  ((x)%BITS_PER_LONG)
 #define BIT(x)  (1UL<<OFF(x))
 #define LONG(x) ((x)/BITS_PER_LONG)
@@ -101,10 +102,10 @@ static const struct query_mode {
 	int max;
 	int rq;
 } query_modes[] = {
-	{ "EV_KEY", EV_KEY, KEY_MAX, EVIOCGKEY(KEY_MAX) },
-	{ "EV_LED", EV_LED, LED_MAX, EVIOCGLED(LED_MAX) },
-	{ "EV_SND", EV_SND, SND_MAX, EVIOCGSND(SND_MAX) },
-	{ "EV_SW",  EV_SW, SW_MAX, EVIOCGSW(SW_MAX) },
+	{ "EV_KEY", EV_KEY, KEY_MAX, EVIOCGKEY(NBYTES(KEY_MAX)) },
+	{ "EV_LED", EV_LED, LED_MAX, EVIOCGLED(NBYTES(LED_MAX)) },
+	{ "EV_SND", EV_SND, SND_MAX, EVIOCGSND(NBYTES(SND_MAX)) },
+	{ "EV_SW",  EV_SW, SW_MAX, EVIOCGSW(NBYTES(SW_MAX)) },
 };
 
 static int grab_flag = 0;
@@ -406,6 +407,11 @@ static const char * const keys[KEY_MAX + 1] = {
 #endif
 #ifdef KEY_IMAGES
 	NAME_ELEMENT(KEY_IMAGES),
+#endif
+#ifdef KEY_NOTIFICATION_CENTER
+	NAME_ELEMENT(KEY_NOTIFICATION_CENTER),
+	NAME_ELEMENT(KEY_PICKUP_PHONE),
+	NAME_ELEMENT(KEY_HANGUP_PHONE),
 #endif
 	NAME_ELEMENT(KEY_DEL_EOL),		NAME_ELEMENT(KEY_DEL_EOS),
 	NAME_ELEMENT(KEY_INS_LINE),	 	NAME_ELEMENT(KEY_DEL_LINE),
@@ -420,6 +426,9 @@ static const char * const keys[KEY_MAX + 1] = {
 	NAME_ELEMENT(KEY_FN_D),			NAME_ELEMENT(KEY_FN_E),
 	NAME_ELEMENT(KEY_FN_F),			NAME_ELEMENT(KEY_FN_S),
 	NAME_ELEMENT(KEY_FN_B),
+#ifdef KEY_FN_RIGHT_SHIFT
+	NAME_ELEMENT(KEY_FN_RIGHT_SHIFT),
+#endif
 	NAME_ELEMENT(KEY_BRL_DOT1),		NAME_ELEMENT(KEY_BRL_DOT2),
 	NAME_ELEMENT(KEY_BRL_DOT3),		NAME_ELEMENT(KEY_BRL_DOT4),
 	NAME_ELEMENT(KEY_BRL_DOT5),		NAME_ELEMENT(KEY_BRL_DOT6),
@@ -433,6 +442,10 @@ static const char * const keys[KEY_MAX + 1] = {
 	NAME_ELEMENT(KEY_NUMERIC_8),		NAME_ELEMENT(KEY_NUMERIC_9),
 	NAME_ELEMENT(KEY_NUMERIC_STAR),		NAME_ELEMENT(KEY_NUMERIC_POUND),
 #endif
+#ifdef KEY_NUMERIC_A
+	NAME_ELEMENT(KEY_NUMERIC_A),		NAME_ELEMENT(KEY_NUMERIC_B),
+	NAME_ELEMENT(KEY_NUMERIC_C),		NAME_ELEMENT(KEY_NUMERIC_D),
+#endif
 #ifdef KEY_NUMERIC_11
 	NAME_ELEMENT(KEY_NUMERIC_11),		NAME_ELEMENT(KEY_NUMERIC_12),
 #endif
@@ -545,6 +558,12 @@ static const char * const keys[KEY_MAX + 1] = {
 #ifdef KEY_ALS_TOGGLE
 	NAME_ELEMENT(KEY_ALS_TOGGLE),
 #endif
+#ifdef KEY_ROTATE_LOCK_TOGGLE
+	NAME_ELEMENT(KEY_ROTATE_LOCK_TOGGLE),
+#endif
+#ifdef KEY_REFRESH_RATE_TOGGLE
+	NAME_ELEMENT(KEY_REFRESH_RATE_TOGGLE),
+#endif
 #ifdef KEY_BUTTONCONFIG
 	NAME_ELEMENT(KEY_BUTTONCONFIG),
 #endif
@@ -569,6 +588,20 @@ static const char * const keys[KEY_MAX + 1] = {
 #ifdef KEY_ASSISTANT
 	NAME_ELEMENT(KEY_ASSISTANT),
 #endif
+#ifdef KEY_EMOJI_PICKER
+	NAME_ELEMENT(KEY_EMOJI_PICKER),
+#endif
+#ifdef KEY_KBD_LAYOUT_NEXT
+	NAME_ELEMENT(KEY_KBD_LAYOUT_NEXT),
+#endif
+#ifdef KEY_DICTATE
+	NAME_ELEMENT(KEY_DICTATE),
+#endif
+#ifdef KEY_CAMERA_ACCESS_ENABLE
+	NAME_ELEMENT(KEY_CAMERA_ACCESS_ENABLE),
+	NAME_ELEMENT(KEY_CAMERA_ACCESS_DISABLE),
+	NAME_ELEMENT(KEY_CAMERA_ACCESS_TOGGLE),
+#endif
 #ifdef KEY_BRIGHTNESS_MIN
 	NAME_ELEMENT(KEY_BRIGHTNESS_MIN),
 #endif
@@ -621,6 +654,69 @@ static const char * const keys[KEY_MAX + 1] = {
 	NAME_ELEMENT(KEY_ONSCREEN_KEYBOARD),
 #endif
 
+#ifdef KEY_PRIVACY_SCREEN_TOGGLE
+	NAME_ELEMENT(KEY_PRIVACY_SCREEN_TOGGLE),
+#endif
+
+#ifdef KEY_SELECTIVE_SCREENSHOT
+	NAME_ELEMENT(KEY_SELECTIVE_SCREENSHOT),
+#endif
+
+#ifdef KEY_NEXT_ELEMENT
+	NAME_ELEMENT(KEY_NEXT_ELEMENT),
+	NAME_ELEMENT(KEY_PREVIOUS_ELEMENT),
+	NAME_ELEMENT(KEY_AUTOPILOT_ENGAGE_TOGGLE),
+	NAME_ELEMENT(KEY_MARK_WAYPOINT),
+	NAME_ELEMENT(KEY_SOS),
+	NAME_ELEMENT(KEY_NAV_CHART),
+	NAME_ELEMENT(KEY_FISHING_CHART),
+	NAME_ELEMENT(KEY_SINGLE_RANGE_RADAR),
+	NAME_ELEMENT(KEY_DUAL_RANGE_RADAR),
+	NAME_ELEMENT(KEY_RADAR_OVERLAY),
+	NAME_ELEMENT(KEY_TRADITIONAL_SONAR),
+	NAME_ELEMENT(KEY_CLEARVU_SONAR),
+	NAME_ELEMENT(KEY_SIDEVU_SONAR),
+	NAME_ELEMENT(KEY_NAV_INFO),
+	NAME_ELEMENT(KEY_BRIGHTNESS_MENU),
+#endif
+
+#ifdef KEY_MACRO1
+	NAME_ELEMENT(KEY_MACRO1),		NAME_ELEMENT(KEY_MACRO2),
+	NAME_ELEMENT(KEY_MACRO3),		NAME_ELEMENT(KEY_MACRO4),
+	NAME_ELEMENT(KEY_MACRO5),		NAME_ELEMENT(KEY_MACRO6),
+	NAME_ELEMENT(KEY_MACRO7),		NAME_ELEMENT(KEY_MACRO8),
+	NAME_ELEMENT(KEY_MACRO9),		NAME_ELEMENT(KEY_MACRO10),
+	NAME_ELEMENT(KEY_MACRO11),		NAME_ELEMENT(KEY_MACRO12),
+	NAME_ELEMENT(KEY_MACRO13),		NAME_ELEMENT(KEY_MACRO14),
+	NAME_ELEMENT(KEY_MACRO15),		NAME_ELEMENT(KEY_MACRO16),
+	NAME_ELEMENT(KEY_MACRO17),		NAME_ELEMENT(KEY_MACRO18),
+	NAME_ELEMENT(KEY_MACRO19),		NAME_ELEMENT(KEY_MACRO20),
+	NAME_ELEMENT(KEY_MACRO21),		NAME_ELEMENT(KEY_MACRO22),
+	NAME_ELEMENT(KEY_MACRO23),		NAME_ELEMENT(KEY_MACRO24),
+	NAME_ELEMENT(KEY_MACRO25),		NAME_ELEMENT(KEY_MACRO26),
+	NAME_ELEMENT(KEY_MACRO27),		NAME_ELEMENT(KEY_MACRO28),
+	NAME_ELEMENT(KEY_MACRO29),		NAME_ELEMENT(KEY_MACRO30),
+#endif
+
+#ifdef KEY_MACRO_RECORD_START
+	NAME_ELEMENT(KEY_MACRO_RECORD_START),
+	NAME_ELEMENT(KEY_MACRO_RECORD_STOP),
+	NAME_ELEMENT(KEY_MACRO_PRESET_CYCLE),
+	NAME_ELEMENT(KEY_MACRO_PRESET1),
+	NAME_ELEMENT(KEY_MACRO_PRESET2),
+	NAME_ELEMENT(KEY_MACRO_PRESET3),
+#endif
+
+#ifdef KEY_KBD_LCD_MENU1
+	NAME_ELEMENT(KEY_KBD_LCD_MENU1),	NAME_ELEMENT(KEY_KBD_LCD_MENU2),
+	NAME_ELEMENT(KEY_KBD_LCD_MENU2),	NAME_ELEMENT(KEY_KBD_LCD_MENU3),
+	NAME_ELEMENT(KEY_KBD_LCD_MENU4),	NAME_ELEMENT(KEY_KBD_LCD_MENU5),
+#endif
+
+#ifdef KEY_PERFORMANCE
+        NAME_ELEMENT(KEY_PERFORMANCE),
+#endif
+
 #ifdef BTN_TRIGGER_HAPPY
 	NAME_ELEMENT(BTN_TRIGGER_HAPPY1),	NAME_ELEMENT(BTN_TRIGGER_HAPPY11),
 	NAME_ELEMENT(BTN_TRIGGER_HAPPY2),	NAME_ELEMENT(BTN_TRIGGER_HAPPY12),
@@ -792,6 +888,9 @@ static const char * const switches[SW_MAX + 1] = {
 #ifdef SW_PEN_INSERTED
 	NAME_ELEMENT(SW_PEN_INSERTED),
 #endif
+#ifdef SW_MACHINE_COVER
+	NAME_ELEMENT(SW_MACHINE_COVER),
+#endif
 };
 
 static const char * const force[FF_MAX + 1] = {
@@ -1069,7 +1168,7 @@ static int print_device_info(int fd)
 	printf("Input device name: \"%s\"\n", name);
 
 	memset(bit, 0, sizeof(bit));
-	ioctl(fd, EVIOCGBIT(0, EV_MAX), bit[0]);
+	ioctl(fd, EVIOCGBIT(0, NBYTES(KEY_MAX)), bit[0]);
 	printf("Supported events:\n");
 
 	for (type = 0; type < EV_MAX; type++) {
@@ -1078,7 +1177,7 @@ static int print_device_info(int fd)
 
 			printf("  Event type %d (%s)\n", type, typename(type));
 			if (type == EV_SYN) continue;
-			ioctl(fd, EVIOCGBIT(type, KEY_MAX), bit[type]);
+			ioctl(fd, EVIOCGBIT(type, NBYTES(KEY_MAX)), bit[type]);
 			for (code = 0; code < KEY_MAX; code++)
 				if (test_bit(code, bit[type])) {
 					if (have_state) {
